from __future__ import unicode_literals

import logging
logger = logging.getLogger('ddusers')

import os
import uuid
from dcxml import simpledc
from PIL import Image
from weasyprint import HTML

from django.db import models
import django.contrib.auth.models 
from django.contrib.auth.models import AbstractUser
from django.contrib.auth.models import Group as AuthGroup
from django_date_extensions.fields import ApproximateDateField
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string

from django.forms import ModelForm
from django.urls import reverse
from django.contrib.contenttypes.fields import GenericRelation

from taggit.managers import TaggableManager
from taggit.models import TagBase, GenericTaggedItemBase

from ddusers.cropped_thumbnail import cropped_thumbnail

from ddpublications.models import Publication, Publisher, Person, Place
from ddpublications.lookup_doi import lookup_doi

from ddcomments.models import Comment

from core.models import KeywordTags
from core.models import EnglishTags
from core.models import AffiliationTags
from core.models import Topic

from django_comments.abstracts import CommentAbstractModel

from guardian.shortcuts import assign_perm, get_users_with_perms, get_perms, remove_perm, get_objects_for_user

from actstream.models import user_stream


def get_users_in_group(self):
    return User.objects.filter(groups=self)

AuthGroup.add_to_class('get_users_in_group', get_users_in_group)






class Group(AuthGroup):
    class Meta:
        proxy = True
    # add method to Group model


    


ACCESS_RULES_CHOICES = (
    ('PUB', 'public'),
    ('NET', 'network'),
    ('GRP', 'group'),
    ('HID', 'hidden'),
)

ACADEMIC_TITLE_CHOICES = (
    ('PHD', 'PhD'),
    ('DR', 'Dr.'),
    ('PROF', 'Prof. Dr.'),
    ('MA', 'MA',),
    ('BA', 'BA',),
    ('MAG', 'Magister'),
    ('M.A.', 'M.A.'),
)

ACADEMIC_TITLE_CHOICES_ORDER = {
    'PHD': 'b',
    'DR': 'f',
    'PROF': 'f',
    'MA': 'b',
    'BA': 'b',
    'MAG':'f',
    'M.A.': 'b',
}


class User(AbstractUser):
    id = models.AutoField(primary_key=True)
    uuid = models.UUIDField(default=uuid.uuid4, editable=False) # uuid _not_ as pk as this disturbs django 3rd party apps
    middle_name = models.CharField(max_length=200, blank=True)
    academic_title = models.CharField(max_length=5, choices=ACADEMIC_TITLE_CHOICES, blank='None') # deprecated
    name_prefix = models.CharField(max_length=200, blank=True, verbose_name='prefix', help_text='for example \"Dr\"')
    name_suffix = models.CharField(max_length=200, blank=True, verbose_name='suffix', help_text='for example \"PhD\"')
    #position = models.CharField(max_length=600)
    #institution = models.ForeignKey('Institution', blank=True, null=True, related_name='user_profile_institution')
    #institutions = models.ManyToManyField('Institution', blank=True,)
    photo = models.ImageField(blank=True, null=True)
    #fields_of_interest = models.ManyToManyField('FOI', blank=True)
    topics = models.ManyToManyField(Topic, blank=True)
    interests = TaggableManager(blank=True, through=KeywordTags)
    receptive_tos = models.ManyToManyField('ReceptiveTo', blank=True)
    countries = models.ManyToManyField('Country', related_name='user_country', blank=True)
    research_profile_full = models.TextField(blank=True, verbose_name='Research Profile (full)')
    research_profile_short = models.TextField(max_length=280, blank=True, verbose_name='Research Profile (brief)', help_text='brief version, max. 280 characters')
    access = models.CharField(max_length=3, choices=ACCESS_RULES_CHOICES, default='PUB')
    publications = models.ManyToManyField('ddpublications.Publication', blank=True, related_name='user_publication')
    en_tags = TaggableManager(blank=True, through=EnglishTags)
    affiliation_tags = TaggableManager("affiliations", blank=True, through=AffiliationTags)
    external_profile = models.URLField(blank=True)

    class Meta:
        ordering = ['last_name']

    def get_academic_name(self):
        a_n = list()
        if self.name_prefix:
            a_n.append(self.name_prefix)

        if self.first_name:
            a_n.append(self.first_name)

        if self.middle_name:
            a_n.append(self.middle_name)

        if self.last_name:
            a_n.append(self.last_name)

        academic_name = ' '.join(a_n)

        if self.name_suffix:
            academic_name = academic_name + ", " + self.name_suffix

        return academic_name


    def get_countries(self):
        return self.countries.all().order_by('name')

    def get_interests(self):
        return self.interests.all().order_by('name')

    def get_main_affiliation(self):
        return Affiliation.objects.filter(user=self).filter(main=True)

    def get_affiliations(self):
        return Affiliation.objects.filter(user=self).order_by('-main')

    #def get_datasets(self):
    #    datasets = get_objects_for_user(self, 'view_dataset')
    #    return datasets #DataSet.objects.filter(user=self).order_by('title')

    def get_projects(self):
        return Project.objects.filter(user=self).order_by('name')
        #projects_list = list()
        #for project in Project.objects.filter(user=self):
        #    projects_list.append(project)
        #return projects_list

    def __str__(self):
        return self.get_academic_name()

    def save(self):
        super(User, self).save()

    def group_member(self, group):
        return self.groups.filter(id=group.id).exists()
    
    @property
    def image_url(self):
        try:        
            return self.thumbnail.url
        except:
            return "/static/images/user_default.png"

    def get_activities(self):
        return user_stream(self, with_user_activity=True)

    # override for thumbnail generation
    def save(self, *args, **kwargs):
        models.Model.save(self, *args, **kwargs)
        try:
            # in situ thumbnail generation at upload
            print('Open image for thumbnail generation')
            im = Image.open(self.photo.path)
            thumbnail = cropped_thumbnail( im, [250,250] )                        
            thumbnail.save(self.photo.path)            
            print('thumbnail saved')
        except:
            print('no image uploaded yet')
            pass
        # Call the "real" save() method in the base class 'models.Model'.
        models.Model.save(self, *args, **kwargs)



class ReceptiveTo(models.Model):
    text = models.CharField(max_length=200, blank=True)
    def __str__(self):
        return self.text


class City(models.Model):
    name = models.CharField(max_length=400)
    slug = models.SlugField()
    geo_lat = models.CharField(max_length=200, blank=True)
    geo_lng = models.CharField(max_length=200, blank=True)
    country = models.ForeignKey('Country', related_name='city_country', on_delete=models.CASCADE)
    public = models.BooleanField(default=True)

    def __str__(self):
        return self.name


class Country(models.Model):
    name = models.CharField(max_length=200)
    slug = models.SlugField()
    geo_lat = models.CharField(max_length=200)
    geo_lng = models.CharField(max_length=200)
    public = models.BooleanField(default=True)

    class Meta:
        ordering = ['name']

    def __str__(self):
        return self.name

class FOI(models.Model):
    name = models.CharField(max_length=200)
    description = models.TextField(blank=True)
    is_country = models.BooleanField(default=False)

    def __str__(self):
        return self.title


class Affiliation(models.Model):
    uuid = models.UUIDField(default=uuid.uuid4, editable=False) # uuid _not_ as pk as this disturbs django 3rd party apps
    user = models.ForeignKey('User', related_name='affiliation_user', on_delete=models.CASCADE)
    institution = models.ForeignKey('Institution', related_name='affiliation_institution', blank=True, null=True, on_delete=models.CASCADE)
    name_of_institution = models.CharField(max_length=200)
    place_of_institution = models.CharField(max_length=200)
    country_of_institution = models.CharField(max_length=200)
    position = models.CharField(max_length=200)
    website_of_institution = models.URLField(blank=True)
    main = models.BooleanField(default=True)

    def __str__(self):
        return "%s, %s, %s, %s" % (self.position, self.name_of_institution, self.place_of_institution, self.country_of_institution)

class Institution(models.Model):
    uuid = models.UUIDField(default=uuid.uuid4, editable=False) # uuid _not_ as pk as this disturbs django 3rd party apps
    name = models.CharField(max_length=600)
    original_name = models.CharField(max_length=600, blank=True)
    short_name = models.CharField(max_length=60, blank=True)
    department = models.CharField(max_length=600, blank=True)
    address = models.CharField(max_length=600)
    postcode = models.CharField(max_length=40, blank=True, null=True)
    image = models.ImageField(blank=True, null=True)
    image_large = models.ImageField(blank=True, null=True)
    website = models.URLField(blank=True)
    description = models.TextField(blank=True)
    short_description = models.CharField(max_length=400, blank=True)
    location = models.CharField(max_length=20, blank=True)
    city = models.CharField(max_length=120)
    country = models.ForeignKey('Country', related_name='institution_country', blank=True, on_delete=models.CASCADE)
    public = models.BooleanField(default=True)
    admin = models.ForeignKey('User', related_name='institution_admin_user', blank=True, null=True, on_delete=models.CASCADE)
    affiliated_users = models.ManyToManyField('User', blank=True, )

    def __str__(self):
        return self.name
    
class Project(models.Model):
    uuid = models.UUIDField(default=uuid.uuid4, editable=False) # uuid _not_ as pk as this disturbs django 3rd party apps
    user = models.ForeignKey('User', related_name='project_user', on_delete=models.CASCADE)
    group = models.ForeignKey(Group, blank=True, null=True, related_name='project_group', on_delete=models.CASCADE)
    name = models.CharField(max_length=200)
    url = models.URLField(blank=True)

    def __str__(self):
        return self.name

