""" Publish to the DARIAH-DE Repository

    Repository Frontend docs:
        https://repository.de.dariah.eu/doc/services/submodules/publikator/docs/

    Usage:

        job = publish.publish_dataset(token=token, dataset_id=1)
        while(True):
            status = job.status()
            # communicate progress to user ... -> status.progress
            if status['finished']:
                break
            time.sleep(1)

"""
import json
import requests

from django.conf import settings
from django.core.files.base import ContentFile
from django.template import loader

from dddatasets.models import DataSet
from dhrep.storage import Storage


class Publish():
    """ Methods for publication to the DARIAH-DE repository
    """

    def __init__(self):
        self._storage = Storage()
        self._publish_url = settings.DARIAH_PUBLISH_URL
        if not self._publish_url.endswith('/'):
            self._publish_url += '/'

    def publish_dataset(self, token, dataset_id):
        """Publish a DiscussData dataset to DARIAH-DE repository, conveninience mathod which calls
        copy_dataset_to_ownstorage() and publish()

        Arguments:
            token {str} -- authentication token
            dataset_id {int} -- ID of DiscussData dataset

        Returns:
            PublishJob -- a PublishJob which contains all relevant information
        """
        job = self.copy_dataset_to_ownstorage(token, dataset_id)
        job.publish()
        return job

    def copy_dataset_to_ownstorage(self, token, dataset_id):
        """Copy all files of a DiscussData dataset to the DARIAH-DE ownstorage and
        create collection rdf with metadata and links to the files. The storage-ID
        of this collection can be used to trigger publication, which could also be
        done with job.publish()

        Arguments:
            token {str} -- authentication token
            dataset_id {int} -- ID of DiscussData dataset

        Returns:
            PublishJob -- a PublishJob which contains all relevant information
        """
        dataset = DataSet.objects.get(id=dataset_id)
        storage_id = self._storage.create(token)

        job = PublishJob(token, storage_id, dataset_id)

        datafiles = dataset.get_datafiles()
        for datafile in datafiles:
            # add storage_ids to datafile objects for
            # usage with create_collection_rdf()
            datafile_storage_id = self.upload_datafile(token, datafile)
            job.files[datafile_storage_id] = {
                'id': datafile.id,
                'dataset_id': dataset_id,
                'name': datafile.name,
                'content_type': datafile.content_type,
                'storage_id': datafile_storage_id,
            }
        turtle = self.create_collection_rdf(storage_id, dataset, job.files)
        tfile = ContentFile(turtle)
        tfile.file.content_type = 'text/plain'
        self._storage.update(token, storage_id, tfile)
        return job

    def publish(self, token, storage_id):
        """Publish a collection where the collection metadata rdf  is located
        at the storage_id.

        Arguments:
            token {str} -- authentication token
            storage_id {str} -- storage_id of collection rdf

        Raises:
            PublishException: An error from publicator if HTTP-Status != 200
        """
        response = requests.post(self._publish_url + storage_id + '/publish',
                                 headers={
                                     'X-Storage-Token': token,
                                     'Accept': 'application/json',
                                     'X-Transaction-ID': 'BLUBBER'
                                 })

        if response.status_code != 200:
            raise PublishException('Error starting publication process: ' + response.text
                                   + ' - ' + str(response.status_code))

    def status(self, token, storage_id):
        """get status from publish service for a given storage-id in publication pipeline.
        better use job.status()

        Arguments:
            token {str} -- authentication token
            storage_id {str} -- storage_id of collection rdf

        Raises:
            PublishException: An error from publicator if HTTP-Status != 200

        Returns:
            JSON -- publish status response as JSON object
        """
        response = requests.get(self._publish_url + storage_id + '/status',
                                headers={'X-Storage-Token': token, 'Accept': 'application/json'},
                                )

        print('status-text: ' + response.text)
        print('status-status: ' + str(response.status_code))

        if response.status_code != 200:
            raise PublishException('Error with publish status: ' + response.text
                                   + ' - ' + str(response.status_code))

        return json.loads(response.text)

    def upload_datafile(self, token, datafile):
        """Upload a Django File to DARIAH-DE ownstorage

        Arguments:
            token {str} -- authentication token
            datafile {File} -- File object from Django

        Returns:
            str -- storage_id from DARIAH-DE ownstorage
        """
        tfile = ContentFile(datafile.file.read())
        # TODO: possible without copying into memory?
        tfile.file.content_type = datafile.content_type
        storage_id = self._storage.create(token, tfile)
        return storage_id

    @staticmethod
    def create_collection_rdf(storage_id, dataset, datafiles):
        """Create dariahrep collection rdf for a dataset

        https://repository.de.dariah.eu/doc/services/submodules/kolibri/kolibri-dhpublish-service/docs/index.html

        Arguments:
            storage_id {str} -- dariahstorage id for the collection file, for self-reference
            dataset {ddusers.model.DataSet} -- a ddusers DataSet to generate rdf for
            datafiles {Array ddusers.model.DataFile} -- Array of ddusers.DataFile contained in
                                                        collection extended with their storage_id

        Returns:
            [str] -- RDF (turtle) to represent the given dataset
        """
        turtle = loader.render_to_string('collection.ttl',
                                         {
                                             'storage_id': storage_id,
                                             'dataset': dataset,
                                             'datafiles': datafiles
                                         })

        return turtle

    def get_publish_url(self):
        """Get the publish service url

        Returns:
            str -- the actual publish service url set
        """
        return self._publish_url


class PublishException(Exception):
    """Thrown in case of problems with the publish service"""


class PublishJob():
    """PublishJob - keep track of status, files and their DOIs in a
    Publish Process to the DARIAH-DE Repository
    """

    def __init__(self, token, collection_id, dataset_id):
        self.token = token
        self.collection_id = collection_id
        self.dataset_id = dataset_id
        self.files = dict()
        self.pid = ''

    def remote_status(self):
        """Get the status of this PublishJob from DARIAH-DE publish service

        Raises:
            PublishException: An error from publicator if HTTP-Status != 200
        Returns:

            JSON -- publish status response as JSON object
        """
        return publish.status(self.token, self.collection_id)

    def publish(self):
        """Inititate publish process for this PublishJob

        Raises:
            PublishException: An error from publicator if HTTP-Status != 200
        """
        return publish.publish(self.token, self.collection_id)

    def status(self):
        """ Status of this job this includes:
            - files, storage-IDs and DOIS if already available
            - progress (in percent)
            - finished (true / false)
            - publish-status answer from remote server

        Returns:
            dict -- finished/progress/file-mappings, ...
        """

        status = self.remote_status()

        # attach pid to files
        if 'publishObjects' in status:
            for pobject in status['publishObjects']:
                # cut 'https://de.dariah.eu/storage/' from uri to get the storage_id
                storage_id = pobject['uri'][29:]
                if 'pid' in pobject:
                    if storage_id == self.collection_id:
                        self.pid = pobject['pid']
                    else:
                        self.files[storage_id]['pid'] = pobject['pid']

        return {
            'finished': status['publishStatus']['processStatus'] == 'FINISHED',
            'progress': status['publishStatus']['progress'],
            'collection': {
                'dataset_id': self.dataset_id,
                'storage_id': self.collection_id,
                'pid': self.pid
            },
            'files': self.files,
            'remote_status': status
        }


# an instance of publish
publish = Publish()
