""" Connection to DARIAH-DE storage

    Storage API docs: http://hdl.handle.net/11858/00-1734-0000-0009-FEA1-D
"""
from django.conf import settings
import requests


class Storage():
    """
    Methods to create and update data objects in DARIAH-DE storage
    """

    def __init__(self):
        storage_url = settings.DARIAH_STORAGE_LOCATION
        if not storage_url.endswith('/'):
            storage_url += '/'
        self._storage_url = storage_url

    def create(self, token, content=None):
        """Create a new object in dariahstorage

        Arguments:
            token {str} -- authentication token
            content {ContentFile, optional} -- data to store (optionally)

        Raises:
            Exception -- if response from storage had no 201 status code

        Returns:
            str -- the storage id of created object
        """

        if content is not None:
            data = content.read()
        else:
            data = None

        response = requests.post(
            self._storage_url,
            headers={
                'Authorization': 'bearer ' + token,
                'Content-type': 'text/plain'
            },
            data=data
        )

        if response.status_code != 201:
            raise StorageException('Error creating new cdstar object: ' + response.text
                                   + ' - ' + str(response.status_code))

        storage_id = response.headers['location'].rsplit('/', 1)[-1]
        return storage_id

    def update(self, token, storage_id, content):
        """Update an object in dariahstorage

        Arguments:
            token {str} -- authentication token
            storageid {str} -- the storage id
            content {ContentFile} -- the data to store

        Raises:
            Exception -- if response from storage had no 201 status code
        """

        response = requests.put(
            self._storage_url + storage_id,
            headers={
                'Authorization': 'bearer ' + token,
                'Content-type': content.file.content_type
            },
            data=content.read()
        )

        if response.status_code != 201:
            raise StorageException(
                'Error updating cdstar object ' + storage_id + ': ' + response.text
                + ' - ' + str(response.status_code)
            )

    def get_storage_url(self):
        """Get the storage url

        Returns:
            str -- the actual storage url set
        """

        return self._storage_url


class StorageException(Exception):
    """Thrown in case of problems with the storage"""


# an instance of storage
storage = Storage()
